#!/usr/bin/env python3

from typing import Iterator, Literal, Mapping, Sequence

from .dfs import depth_first_search
from .modules import Module
from .tarjan import strongly_connected_components


def detect_cycles(
    strategy: Literal["dfs", "tarjan"],
    graph: Mapping[Module, Sequence[Module]],
) -> Iterator[tuple[Module, ...]]:
    if strategy == "dfs":
        return depth_first_search(graph)
    if strategy == "tarjan":
        return (scc for scc in strongly_connected_components(graph) if len(scc) > 1)
    raise NotImplementedError()



#!/usr/bin/env python3

import logging
import sys
from pathlib import Path

logger = logging.getLogger(__name__)


def setup_logging(filepath: Path, opt_debug: bool) -> None:
    filepath.unlink(missing_ok=True)

    if opt_debug:
        sys.stderr.write(f"Write log to {filepath}\n")
        log_level = logging.DEBUG
    else:
        log_level = logging.NOTSET

    logger.setLevel(log_level)

    handler = logging.FileHandler(filename=filepath)
    handler.setLevel(log_level)

    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
    handler.setFormatter(formatter)
    logger.addHandler(handler)



#!/usr/bin/env python3

from __future__ import annotations

import abc
from typing import Protocol, TypeVar


class Comparable(Protocol):
    @abc.abstractmethod
    def __lt__(self: T, other: T) -> bool:
        ...


T = TypeVar("T", bound=Comparable)



#!/usr/bin/env python3

from typing import Iterator, List, Mapping, Sequence, Set, TypeVar

from .type_defs import Comparable

T = TypeVar("T", bound=Comparable)


def depth_first_search(graph: Mapping[T, Sequence[T]]) -> Iterator[tuple[T, ...]]:
    visited: Set[T] = set()

    def _dfs_util(vertex_u: T, path: List[T]) -> Iterator[tuple[T, ...]]:
        if vertex_u in visited:
            return

        for vertex_v in graph.get(vertex_u, []):
            if vertex_v in path:
                yield tuple(path[path.index(vertex_v) :])
                continue

            yield from _dfs_util(vertex_v, path + [vertex_v])

        visited.add(vertex_u)

    for vertex in sorted(graph):
        yield from _dfs_util(vertex, [])
